@@ -33,58 +33,17 @@
 
 #include "base/base.h"
 #include "base/version.h"
+#include "unix/ibus/main.h"
 #include "unix/ibus/mozc_engine.h"
 #include "unix/ibus/path_util.h"
 
 DEFINE_bool(ibus, false, "The engine is started by ibus-daemon");
-DEFINE_bool(xml, false, "Dump supported engines as XML");
-
-#ifdef OS_CHROMEOS
-static const bool kEnableUsEngineDefault = true;
-#else
-static const bool kEnableUsEngineDefault = false;
-#endif
-DEFINE_bool(enable_us_engine,
-            kEnableUsEngineDefault, "Enable Mozc engine for a US keyboard");
 
 namespace {
 
 IBusBus *g_bus = NULL;
 IBusConfig *g_config = NULL;
 
-const char kComponentName[] = "com.google.IBus.Mozc";
-const char kComponentDescription[] = "Mozc Component";
-const char kLicense[] = "New BSD";
-const char kAuthor[] = "Google Inc.";
-const char kHomePage[] = "http://code.google.com/p/mozc/";
-const char kTextDomain[] = "ibus-mozc";
-#ifdef GOOGLE_JAPANESE_INPUT_BUILD
-const char *kEngineLongNames[] = {
-  "Google Japanese Input (US keyboard layout)",
-  "Google Japanese Input (Japanese keyboard layout)",
-};
-#else
-const char *kEngineLongNames[] = {
-  "Mozc (US keyboard layout)",
-#ifdef OS_CHROMEOS
-  "Mozc (Japanese keyboard layout)",
-#else
-  "Mozc",
-#endif  // OS_CHROMEOS
-};
-#endif  // GOOGLE_JAPANESE_INPUT_BUILD
-const char kEngineDescription[] = "Mozc Japanese input method";
-const char kEngineIcon[] = "product_icon.png";
-const char kEngineLanguage[] = "ja";
-#ifdef OS_CHROMEOS
-// TODO(yusukes): Check if Kana key in the Chrome OS Japanese keyboard actually
-// generates the Hiragana_Katakana keysym on "jp" XKB layout when the keyboard
-// gets ready.
-const char kEngineHotkeys[] = "Hiragana_Katakana";
-#endif
-const size_t kEngineNamesLen = arraysize(mozc::ibus::kEngineNames);
-COMPILE_ASSERT(kEngineNamesLen == arraysize(kEngineLongNames), bad_array_size);
-
 #ifndef OS_CHROMEOS
 void IgnoreSigChild() {
   // Don't wait() child process termination.
@@ -103,45 +62,25 @@ IBusComponent *GetIBusComponent() {
       kComponentName,
       kComponentDescription,
       mozc::Version::GetMozcVersion().c_str(),
-      kLicense,
-      kAuthor,
-      kHomePage,
+      kComponentLicense,
+      kComponentAuthor,
+      kComponentHomepage,
       "",
-      kTextDomain);
-  const string &icon_path = mozc::ibus::GetIconPath(kEngineIcon);
+      kComponentTextdomain);
+  const string icon_path = mozc::ibus::GetIconPath(kEngineIcon);
 
-  for (size_t i = 0; i < kEngineNamesLen; ++i) {
-    if (!FLAGS_enable_us_engine &&
-        !strcmp(mozc::ibus::kEngineLayouts[i], "us")) {
-      continue;
-    }
-
-    ibus_component_add_engine(
-        component,
-#ifdef OS_CHROMEOS
-        // On Chrome OS, we use "Kana" key for selecting mozc or mozc-jp engines
-        // unconditionally (i.e. no-op if mozc is already active).
-        // Note that the ibus_engine_desc_new2 API is Chrome OS specific as of
-        // writing.
-        ibus_engine_desc_new2(mozc::ibus::kEngineNames[i],
-                              kEngineLongNames[i],
-                              kEngineDescription,
-                              kEngineLanguage,
-                              kLicense,
-                              kAuthor,
-                              icon_path.c_str(),
-                              mozc::ibus::kEngineLayouts[i],
-                              kEngineHotkeys));
-#else
-        ibus_engine_desc_new(mozc::ibus::kEngineNames[i],
-                             kEngineLongNames[i],
-                             kEngineDescription,
-                             kEngineLanguage,
-                             kLicense,
-                             kAuthor,
-                             icon_path.c_str(),
-                             mozc::ibus::kEngineLayouts[i]));
-#endif
+  // The engine description(s) in main.h is used only when --ibus option is not
+  // specified (i.e. debugging).
+  for (size_t i = 0; i < kEngineArrayLen; ++i) {
+    ibus_component_add_engine(component,
+                              ibus_engine_desc_new(kEngineNameArray[i],
+                                                   kEngineLongnameArray[i],
+                                                   kEngineDescription,
+                                                   kEngineLanguage,
+                                                   kComponentLicense,
+                                                   kComponentAuthor,
+                                                   icon_path.c_str(),
+                                                   kEngineLayoutArray[i]));
   }
   return component;
 }
@@ -182,21 +121,13 @@ void InitIBusComponent(bool executed_by_ibus_daemon) {
 int main(gint argc, gchar **argv) {
   InitGoogle(argv[0], &argc, &argv, true);
   ibus_init();
-  if (FLAGS_xml) {
-    GString *xml = g_string_new("");
-    IBusComponent *component = GetIBusComponent();
-    ibus_component_output_engines(component, xml, 0);
-    printf("%s", xml->str);
-    fflush(stdout);
-  } else {
-    InitIBusComponent(FLAGS_ibus);
+  InitIBusComponent(FLAGS_ibus);
 #ifndef OS_CHROMEOS
-    IgnoreSigChild();
+  IgnoreSigChild();
 #endif
-    ibus_main();
-    if (g_config) {
-      g_object_unref(g_config);
-    }
+  ibus_main();
+  if (g_config) {
+    g_object_unref(g_config);
   }
   return 0;
 }
